#
#  Name:    MpNum.py
#
#  Purpose: Python module interface to MX using NumPy arrays.
#
#  Author:  William Lavender
#
#-----------------------------------------------------------------------
#
#  Copyright 2006, 2009, 2011 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

#
# Import NumPy if it has not yet been imported by another package
# such as matplotlib.
#

import sys
import numpy
import Mp
import MpPrivate

#----------------------------------------------------------------------------

def _get_numpy_type_from_mx_type( mx_datatype ):
	if ( mx_datatype == 1 ):
		return "str_"

	elif ( mx_datatype == 2 ):
		return "int8"

	elif ( mx_datatype == 3 ):
		return "uint8"

	elif ( mx_datatype == 4 ):
		return "int16"

	elif ( mx_datatype == 5 ):
		return "uint16"

	elif ( mx_datatype == 6 ):
		return "int32"

	# MX datatype 7 is reserved.

	# FIXME: We need to check for 64-bit machines here.

	elif ( mx_datatype == 8 ):
		return "int32"

	elif ( mx_datatype == 9 ):
		return "uint32"

	elif ( mx_datatype == 10 ):
		return "float32"

	elif ( mx_datatype == 11 ):
		return "float64"

	elif ( mx_datatype == 12 ):
		return "uint32"      # WARNING: This must match type 9!

	# MX datatype 13 is reserved.

	elif ( mx_datatype == 14 ):
		return "int64"

	elif ( mx_datatype == 15 ):
		return "uint64"

	else:
		msg = "Unsupported MX type %d requested." % mx_datatype
		raise TypeError, msg

#----------------------------------------------------------------------------

#
#  Net class - Used to access individual network fields in a remote server
#              using NumPy arrays.
#

class Net(Mp.Net):

	def get( self ):
		if ( hasattr( self, "datatype" ) == 0 ):
			self.get_field_parameters()

			self.numpy_type = \
				_get_numpy_type_from_mx_type( self.datatype )

		binary_string = \
			MpPrivate.net_get_remote_binary_string( self.handle )

		array = numpy.fromstring( binary_string, dtype=self.numpy_type )

		shaped_array = numpy.reshape( array, self.dimension_tuple )

		return shaped_array

#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

	def put( self, args ):
		if ( hasattr( self, "datatype" ) == 0 ):
			self.get_field_parameters()

			self.numpy_type = \
				_get_numpy_type_from_mx_type( self.datatype )

		try:
			# Will succeed if 'args' is a NumPy array.
			#
			# FIXME: Not yet really tested.

			binary_string = args.tostring()
		except:
			# We get here if 'args' is not a NumPy array.

			binary_string = Mp._convert_sequence_to_binary_string( \
					args,                       \
					self.datatype,              \
					self.num_dimensions,        \
					self.dimension_tuple,       \
					self.data_element_size_tuple[0], \
					self.name )

		MpPrivate.net_put_remote_binary_string( self.handle, \
							binary_string )

#----------------------------------------------------------------------------

#
#  ImageFrame class - Adds support for interoperating with NumPy arrays.
#

class ImageFrame(Mp.ImageFrame):
	def __init__( self, \
			framesize, image_format, pixel_order, \
			bytes_per_pixel, header_length, image_length ):

		Mp.ImageFrame.__init__( self, \
			framesize, image_format, pixel_order, \
			bytes_per_pixel, header_length, image_length )

		self._class = ImageFrame

		if ( image_format == Mp.MXT_IMAGE_FORMAT_GREY8 ):
			self.numpy_type = "uint8"

		elif ( image_format == Mp.MXT_IMAGE_FORMAT_GREY16 ):
			self.numpy_type = "uint16"

		elif ( image_format == Mp.MXT_IMAGE_FORMAT_GREY32 ):
			self.numpy_type = "uint32"

		else:
			raise Mp.NotYetImplementedError, \
		    "Support for MX image format %d is not yet implemented." \
		    % ( image_format )

	def from_numpy( self, numpy_array ):
		if ( numpy_array.dtype != self.numpy_type ):
			raise Mp.TypeMismatchError, \
	    "The NumPy array datatype is not compatible with this image frame."

		total_pixels = self.framesize[0] * self.framesize[1]

		if ( numpy_array.ndim == 1 ):
			if ( numpy_array.size == total_pixels ):
				image_data = numpy_array.tostring()
			else:
				raise Mp.TypeMismatchError, \
		"The length of the NumPy array does not match " \
			+ "the number of pixels in the ImageFrame."

		elif ( numpy_array.ndim == 2 ):
			if ( numpy_array.shape == self.framesize ):
				flat_numpy_array = \
				    numpy.reshape( numpy_array, total_pixels )

				image_data = flat_numpy_array.tostring()
			else:
				raise Mp.TypeMismatchError, \
		"The shape of the NumPy array does not match " \
			+ "the shape of the ImageFrame."

		else:
			raise Mp.TypeMismatchError, \
		    "NumPy arrays with 3 or more dimensions are not supported."

		self.set_image_data( image_data )

	def to_numpy( self, flat=False ):
		print "self.framesize =", self.framesize

		image_data = self.get_image_data()

		print "len(image_data) =", len(image_data)

		print "self.numpy_type =", self.numpy_type

		numpy_array = \
			numpy.fromstring( image_data, dtype=self.numpy_type )

		print "len(numpy_array) #1 =", len(numpy_array)

		if ( flat == False ):
			numpy_array = \
				numpy.reshape( numpy_array, self.framesize )

		print "len(numpy_array) #2 =", len(numpy_array)

		return numpy_array

