/*
 * Name:    mp_usb.c
 *
 * Purpose: Python module interface to generic MX USB functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2004, 2009, 2011 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "mp.h"

#include "mx_usb.h"

static mx_status_type
MpPrivate_get_usb_device_from_handle( long bus_handle,
					long device_handle,
					MX_USB_DEVICE **usb_device )
{
	static const char fname[] = "MpPrivate_get_usb_device_from_handle()";

	MX_RECORD *usb_record;
	MX_USB *usb;
	void *device_ptr;
	mx_status_type mx_status;

	if ( usb_device == (MX_USB_DEVICE **) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
		"The MX_USB_DEVICE pointer passed was NULL." );
	}

	mx_status = MpPrivate_get_record_from_handle( bus_handle, &usb_record );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	usb = (MX_USB *) usb_record->record_class_struct;

	if ( usb == (MX_USB *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
			"The MX_USB pointer for record '%s' is NULL.",
			usb_record->name );
	}

	mx_status = mx_get_pointer_from_handle( &device_ptr,
						usb->handle_table,
						device_handle );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	*usb_device = (MX_USB_DEVICE *) device_ptr;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT PyObject *
MpPrivate_usb_enumerate_devices_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *usb_record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &usb_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_usb_enumerate_devices( usb_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_usb_find_device_by_order_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *usb_record;
	MX_USB_DEVICE *usb_device;
	unsigned long vendor_id, product_id;
	long handle;
	int parse_status, order_number, configuration_number;
	int interface_number, alternate_interface_number, quiet_flag;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "llliiiii",
				&handle, &vendor_id, &product_id, &order_number,
				&configuration_number, &interface_number,
				&alternate_interface_number, &quiet_flag );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &usb_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_usb_find_device_by_order( usb_record, &usb_device,
					vendor_id, product_id, order_number,
					configuration_number, interface_number,
					alternate_interface_number, quiet_flag);

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_get_handle_from_pointer( &handle,
						usb_device->usb->handle_table,
						usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", usb_device->handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_usb_find_device_by_serial_number_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *usb_record;
	MX_USB_DEVICE *usb_device;
	unsigned long vendor_id, product_id;
	long handle;
	char *serial_number;
	int parse_status, configuration_number;
	int interface_number, alternate_interface_number, quiet_flag;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lllsiiii", &handle,
				&vendor_id, &product_id, &serial_number,
				&configuration_number, &interface_number,
				&alternate_interface_number, &quiet_flag );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &usb_record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_usb_find_device_by_serial_number(usb_record, &usb_device,
					vendor_id, product_id, serial_number,
					configuration_number, interface_number,
					alternate_interface_number, quiet_flag);

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_get_handle_from_pointer( &handle,
						usb_device->usb->handle_table,
						usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", usb_device->handle );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_usb_delete_device_cmd( PyObject *self, PyObject *args )
{
	MX_USB_DEVICE *usb_device;
	long bus_handle, device_handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&bus_handle, &device_handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_usb_device_from_handle( bus_handle,
						device_handle, &usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_usb_delete_device( usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_usb_reset_device_cmd( PyObject *self, PyObject *args )
{
	MX_USB_DEVICE *usb_device;
	long bus_handle, device_handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&bus_handle, &device_handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_usb_device_from_handle( bus_handle,
						device_handle, &usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_usb_reset_device( usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_usb_control_transfer_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_USB_DEVICE *usb_device;
	long bus_handle, device_handle;
	int request_type, request, value, index_or_offset;
	int num_bytes_to_write, num_bytes_to_read;
	int transfer_length, num_bytes_transferred, null_fill_length;
	int parse_status;
	char transfer_buffer[2000];
	char *write_buffer, *ptr;
	double timeout;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lliiiiz#id",
					&bus_handle, &device_handle,
					&request_type, &request,
					&value, &index_or_offset,
					&write_buffer, &num_bytes_to_write,
					&num_bytes_to_read, &timeout );

	if ( parse_status == 0 )
		return NULL;

	/* At this level of code, we do not know whether or not we are
	 * performing a write or a read.  Thus, we must make the best
	 * estimate we can of the value of 'transfer_length'.  The estimate
	 * is only problematic if num_bytes_to_write and num_bytes_to_read
	 * are both non-zero.
	 */

	if ( write_buffer == NULL ) {
		num_bytes_to_write = 0;
	}

	if ( num_bytes_to_write > sizeof( transfer_buffer ) ) {
		MpError_SetString( PyExc_RuntimeError,
		"The number of bytes requested to be written (%ld) is larger "
		"than the transfer buffer size (%ld).",
			num_bytes_to_write, (long) sizeof(transfer_buffer) );

		return NULL;
	}

	if ( num_bytes_to_read > sizeof( transfer_buffer ) ) {
		MpError_SetString( PyExc_RuntimeError,
		"The number of bytes requested to be read (%ld) is larger "
		"than the transfer buffer size (%ld).",
			num_bytes_to_read, (long) sizeof(transfer_buffer) );

		return NULL;
	}

	transfer_length = 0;

	if ( num_bytes_to_write > 0 ) {
		transfer_length = num_bytes_to_write;
	}

	if ( num_bytes_to_read > transfer_length ) {
		transfer_length = num_bytes_to_read;
	}

	if ( write_buffer != NULL ) {
		memcpy( transfer_buffer, write_buffer, num_bytes_to_write );

		/* Null fill any trailing parts of the buffer. */

		null_fill_length = transfer_length - num_bytes_to_write;

		if ( null_fill_length > 0 ) {
			ptr = transfer_buffer + num_bytes_to_write;

			memset( ptr, '\0', null_fill_length );
		}
	}

	mx_status = MpPrivate_get_usb_device_from_handle( bus_handle,
						device_handle, &usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_usb_control_transfer( usb_device,
					request_type, request,
					value, index_or_offset,
					transfer_buffer, transfer_length,
					&num_bytes_transferred, timeout );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* If no data was supposed to be read, return None. */

	if ( num_bytes_to_read == 0 ) {
		Py_INCREF(Py_None);

		return Py_None;
	}

	/* Otherwise, build a string to return to Python. */

	result = Py_BuildValue( "s#", transfer_buffer, num_bytes_transferred );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_usb_bulk_read_cmd( PyObject *self, PyObject *args )
{
	static const char fname[] = "usb_bulk_read";

	PyObject *result;
	MX_USB_DEVICE *usb_device;
	long bus_handle, device_handle;
	int parse_status, endpoint_number, num_bytes_to_read, num_bytes_read;
	double timeout;
	char read_buffer[2000];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lliid",
				&bus_handle, &device_handle, &endpoint_number,
				&num_bytes_to_read, &timeout );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_usb_device_from_handle( bus_handle,
						device_handle, &usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	if ( num_bytes_to_read > sizeof(read_buffer) ) {
		mx_status = mx_error( MXE_WOULD_EXCEED_LIMIT, fname,
		"A bulk read of %d bytes was requested from "
		"USB device (%#lx, %#lx) which would overflow the "
		"internal read buffer of %d bytes.",
			num_bytes_to_read, usb_device->vendor_id,
			usb_device->product_id, (int) sizeof(read_buffer) );

		return MpError_SetFromMxStatus( NULL, mx_status );
	}

	mx_status = mx_usb_bulk_read( usb_device, endpoint_number,
					read_buffer, num_bytes_to_read,
					&num_bytes_read, timeout );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Send the data back to Python as a string object. */

	result = Py_BuildValue( "s#", read_buffer, num_bytes_read );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_usb_bulk_write_cmd( PyObject *self, PyObject *args )
{
	MX_USB_DEVICE *usb_device;
	long bus_handle, device_handle;
	int parse_status, endpoint_number, num_bytes_to_write;
	char *write_buffer;
	double timeout;
	mx_status_type mx_status;

	write_buffer = NULL;

	parse_status = PyArg_ParseTuple( args, "llis#d",
				&bus_handle, &device_handle, &endpoint_number,
				&write_buffer, &num_bytes_to_write,
				&timeout );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_usb_device_from_handle( bus_handle,
						device_handle, &usb_device );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	/* Send the buffer to the USB bus. */ 

	mx_status = mx_usb_bulk_write( usb_device, endpoint_number,
					write_buffer, num_bytes_to_write,
					NULL, timeout );

	if ( mx_status.code != MXE_SUCCESS )
		return MpError_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

